///////////////////////////////////////////////////////////////////////////////
//
//  This software module was originally developed by SAMSUNG ELECTRONICS CO., LTD.
// and Kyunghee University, Korea in the course of development of the ISO/IEC 
// 14496-20(Lightweight Application Scene Representation -LASeR- and Simple 
// Aggregation Format -SAF-) AMD2: Adaptation standard for reference purposes and
// its performance may not have been optimized.
//
// This software module is an implementation of one or more tools as specified by 
// the ISO/IEC 14496-20 AMD2: Adaptation standard.
//
// ISO/IEC gives users free license to this software module or modifications 
// thereof for use in hardware or software products claiming conformance to 
// audiovisual and image-coding related ITU Recommendations and/or ISO/IEC
// International Standards.
//
// ISO/IEC gives users the same free license to this software module or 
// modifications thereof for research purposes and further ISO/IEC standardisation.
// Those intending to use this software module in hardware or software products are
// advised that its use may infringe existing patents. 
//
// The original developer of this software module and his/her company, the 
// subsequent editors and their companies, and ISO/IEC have no liability for use of
// this software module or modifications thereof in an implementation.
//
// Copyright is not released for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
// The organizations named above retain full rights to use the code for their own
// purpose, assign or donate the code to a third party and inhibit third parties 
// from using the code for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
//
// This copyright notice must be included in all copies or derivative works.
// 
// Copyright (c) ISO/IEC 2010
//
// This notice must be included in all copies or derivative works.
//
#include "laser_error.h"
#include "..\svg.h"
#include "..\Datatypes.h"


CSVGAnimation::CSVGAnimation()
{
	CSVGAudio::CSVGAudio();

	m_uElementType			= SVG_animation;

	m_pFocus				= new CSVGFocus();
	m_pLocatable			= new CSVGLocatable();

	m_uFocusHighlight		= 0;
	m_fWidth				= 0.0;
	m_fHeight				= 0.0;
	m_fX					= 0.0;
	m_fY					= 0.0;
	m_uPreserveAspectRatio	= 0;
	m_uInitialVisibility	= 0;
	m_uFill					= 0;
}

CSVGAnimation::~CSVGAnimation()
{
	DELETEMEMORY(m_pFocus);
	DELETEMEMORY(m_pLocatable);
}

/************************************************************************/
// SetAttribute
// Desc		: function of 'animation' to set the children elements or the attributes
// Param	: DOM tree
// History
//	PSH (2008. 8. 15)	- first release
/************************************************************************/
lsrErr CSVGAnimation::SetAttrs(LPXNode pScene, CLASeR *pI)
{
	lsrErr err = LASeRNoErr;

	if (!m_pFocus || !m_pLocatable)
	{
		err = LASeRAnimationErr;
		return err;
	}

	const TCHAR	*szTemp	= NULL;

	/************************************************************************/
	// set attribute
	// svg audio
	err = CSVGAudio::SetAttrs(pScene, pI);
	CONFIRM(err);

	// svg focus
	err = m_pFocus->SetAttrs(pScene, pI);
	CONFIRM(err);

	// svg locatable
	err = m_pLocatable->SetAttrs(pScene, pI);
	CONFIRM(err);

	// focusHighlight
	szTemp = pScene->GetAttrValue(_T("focusHighlight"));
	if (szTemp)
	{
		m_uFocusHighlight = ReadFocusHighlightType(szTemp, &err);
		CONFIRM(err);
	}

	// width
	szTemp = pScene->GetAttrValue(_T("width"));
	if (szTemp)
		m_fWidth = (float)_tstof(szTemp);

	// height
	szTemp = pScene->GetAttrValue(_T("height"));
	if (szTemp)
		m_fHeight = (float)_tstof(szTemp);

	// x
	szTemp = pScene->GetAttrValue(_T("x"));
	if (szTemp)
		m_fX = (float)_tstof(szTemp);

	// y
	szTemp = pScene->GetAttrValue(_T("y"));
	if (szTemp)
		m_fY = (float)_tstof(szTemp);

	// preserveAspectRatio
	szTemp = pScene->GetAttrValue(_T("preserveAspectRatio"));
	if (szTemp)
	{
		m_uPreserveAspectRatio = ReadPreserveAspectRatioType(szTemp, &err);
		CONFIRM(err);
	}

	// initialVisibility
	szTemp = pScene->GetAttrValue(_T("initialVisibility"));
	if (szTemp)
	{
		m_uInitialVisibility = ReadInitialVisibilityType(szTemp, &err);
		CONFIRM(err);
	}

	// fill
	szTemp = pScene->GetAttrValue(_T("fill"));
	if (szTemp)
	{
		m_uFill = ReadAnimFillType(szTemp, &err);
		CONFIRM(err);
	}

	return err;
}

/************************************************************************/
// CheckRigthChild
// Desc		: function to check the right child element
// Param	: element type
// History
//	PSH (2008. 12. 11)	- first release
/************************************************************************/
lsrErr CSVGAnimation::CheckRightChild(UINT8 uElementType)
{
	lsrErr err = LASeRNoErr;

	switch (uElementType)
	{
	case SVG_animate:
	case SVG_animateColor:
	case SVG_animateMotion:
	case SVG_animateTransform:
	case SVG_desc:
	case SVG_discard:
	case SVG_handler:
	case SVG_metadata:
	case SVG_set:
	case SVG_switch:
	case SVG_title:
		/* nothing */
		break;

	default:
		err = LASeRAnimationErr;
		break;
	}

	return err;
}

/************************************************************************/
// GetAttribute
// Desc		: function of 'animation' to get attribute value using it's type
// Param	: attribute type
// History
//	PSH (2008. 9. 11)	- first release
/************************************************************************/
float CSVGAnimation::GetFloatAttr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	float fTemp = 0.0;

	switch (uAttrType)
	{
	case ATTR_WIDTH:
		return m_fWidth;
		break;

	case ATTR_HEIGHT:
		return m_fHeight;
		break;

	case ATTR_X:
		return m_fX;
		break;

	case ATTR_Y:
		return m_fY;
		break;

	default:
		fTemp = CSVGAudio::GetFloatAttr(uAttrType, pErr);
		if (*pErr == LASeRNoErr)	return fTemp;

		*pErr = LASeRAnimationErr;

		break;
	}

	return fTemp;
}

UINT8 CSVGAnimation::GetUINT8Attr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	UINT8 uTemp = 0;

	switch (uAttrType)
	{
	case ATTR_FOCUSHIGHLIGHT:
		return m_uFocusHighlight;
		break;

	case ATTR_PRESERVEASPECTRATIO:
		return m_uPreserveAspectRatio;
		break;

	case ATTR_INITIALVISIBILITY:
		return m_uInitialVisibility;
		break;

	case ATTR_FILL:
		return m_uFill;
		break;

	default:
		uTemp = CSVGAudio::GetUINT8Attr(uAttrType, pErr);
		if (*pErr == LASeRNoErr)	return uTemp;
		else *pErr = LASeRNoErr;

		uTemp = m_pFocus->GetUINT8Attr(uAttrType, pErr);
		if (*pErr == LASeRNoErr)	return uTemp;

		*pErr = LASeRAnimationErr;

		break;
	}

	return uTemp;
}

UINT CSVGAnimation::GetUINTAttr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	UINT uTemp = 0;

	uTemp = CSVGAudio::GetUINTAttr(uAttrType, pErr);
	if (*pErr == LASeRNoErr)	return uTemp;

	*pErr = LASeRAnimationErr;

	return uTemp;
}

tstring CSVGAnimation::GetStrAttr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	tstring strTemp = _T("");

	strTemp = CSVGAudio::GetStrAttr(uAttrType, pErr);
	if (*pErr == LASeRNoErr)	return strTemp;
	else *pErr = LASeRNoErr;

	strTemp = m_pFocus->GetStrAttr(uAttrType, pErr);
	if (*pErr == LASeRNoErr)	return strTemp;
	else *pErr = LASeRNoErr;

	strTemp = m_pLocatable->GetStrAttr(uAttrType, pErr);
	if (*pErr == LASeRNoErr)	return strTemp;

	*pErr = LASeRAnimationErr;

	return strTemp;
}

deque<UINT8> CSVGAnimation::GetUINT8ListAttr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	deque<UINT8> auTemp;

	auTemp = CSVGAudio::GetUINT8ListAttr(uAttrType, pErr);
	if (*pErr == LASeRNoErr)	return auTemp;

	*pErr = LASeRAnimationErr;

	return auTemp;
}

deque<float> CSVGAnimation::GetFloatListAttr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	deque<float> afTemp;

	afTemp = m_pLocatable->GetFloatListAttr(uAttrType, pErr);
	if (*pErr == LASeRNoErr)	return afTemp;

	*pErr = LASeRAnimationErr;

	return afTemp;
}

deque<tstring> CSVGAnimation::GetStrListAttr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	deque<tstring> astrTemp;

	astrTemp = CSVGAudio::GetStrListAttr(uAttrType, pErr);
	if (*pErr == LASeRNoErr)	return astrTemp;

	*pErr = LASeRAnimationErr;

	return astrTemp;
}

/************************************************************************/
// GetAttrType
// Desc         : function of 'animation' attribute group to get attribute type using it's name
// Param        : attribute name
// History
//	PSH (2008. 12. 12)	- first release
/************************************************************************/
UINT CSVGAnimation::GetAttrType(tstring strAttrName)
{
	UINT uTemp = 0;

	if (strAttrName == _T("focusHighlight"))				return ATTR_FOCUSHIGHLIGHT;
	else if (strAttrName == _T("width"))					return ATTR_WIDTH;
	else if (strAttrName == _T("height"))					return ATTR_HEIGHT;
	else if (strAttrName == _T("x"))						return ATTR_X;
	else if (strAttrName == _T("y"))						return ATTR_Y;
	else if (strAttrName == _T("preserveAspectRatio"))		return ATTR_PRESERVEASPECTRATIO;
	else if (strAttrName == _T("initialVisibility"))		return ATTR_INITIALVISIBILITY;
	else if (strAttrName == _T("fill"))						return ATTR_FILL;
	else
	{
		uTemp = CSVGAudio::GetAttrType(strAttrName);
		if (uTemp != 0)	return uTemp;

		uTemp = m_pFocus->GetAttrType(strAttrName);
		if (uTemp != 0)	return uTemp;

		uTemp = m_pLocatable->GetAttrType(strAttrName);
		if (uTemp != 0)	return uTemp;
	}

	return uTemp;
}

/************************************************************************/
// GetAttrValType
// Desc         : function of 'animation' attribute group to get attribute value type using it's type
// Param        : attribute type
// History
//	PSH (2008. 12. 12)       - first release
/************************************************************************/
UINT8 CSVGAnimation::GetAttrValType(UINT uAttrType)
{
	UINT8 uTemp = 0;

	switch (uAttrType)
	{
	case ATTR_FOCUSHIGHLIGHT:
	case ATTR_PRESERVEASPECTRATIO:
	case ATTR_INITIALVISIBILITY:
	case ATTR_FILL:
		return UINT8_TYPE;
		break;

	case ATTR_WIDTH:
	case ATTR_HEIGHT:
	case ATTR_X:
	case ATTR_Y:
		return FLOAT_TYPE;
		break;

	default:
		uTemp = CSVGAudio::GetAttrValType(uAttrType);
		if (uTemp != 0)	return uTemp;

		uTemp = m_pFocus->GetAttrValType(uAttrType);
		if (uTemp != 0)	return uTemp;

		uTemp = m_pLocatable->GetAttrValType(uAttrType);
		if (uTemp != 0)	return uTemp;

		break;
	}

	return uTemp;
}

/************************************************************************/
// SetAttr
// Desc		: function of 'animation' to set attribute value using it's name
// Param	: target attribute name, value
// History
//	PSH (2008. 12. 12)	- first release
/************************************************************************/
lsrErr CSVGAnimation::SetAttr(UINT uAttrType, tstring strValue)
{
	lsrErr err = LASeRNoErr;

	const TCHAR *szValue = strValue.c_str();

	switch (uAttrType)
	{
	case ATTR_WIDTH:
		m_fWidth = (float)_tstof(szValue);
		break;

	case ATTR_HEIGHT:
		m_fHeight = (float)_tstof(szValue);
		break;

	case ATTR_X:
		m_fX = (float)_tstof(szValue);
		break;

	case ATTR_Y:
		m_fY = (float)_tstof(szValue);
		break;

	default:
		err = CSVGAudio::SetAttr(uAttrType, strValue);
		if (err == LASeRNoErr)	return err;
		else err = LASeRNoErr;

		err = m_pFocus->SetAttr(uAttrType, strValue);
		if (err == LASeRNoErr)	return err;
		else err = LASeRNoErr;

		err = m_pLocatable->SetAttr(uAttrType, strValue);
		if (err == LASeRNoErr)	return err;

		err = LASeRAnimationErr;

		break;
	}

	return err;
}

lsrErr CSVGAnimation::SetAttr(UINT uAttrType, float fValue)
{
	lsrErr err = LASeRNoErr;

	switch (uAttrType)
	{
	case ATTR_WIDTH:
		m_fWidth = fValue;
		break;

	case ATTR_HEIGHT:
		m_fHeight = fValue;
		break;

	case ATTR_X:
		m_fX = fValue;
		break;

	case ATTR_Y:
		m_fY = fValue;
		break;

	default:
		err = CSVGAudio::SetAttr(uAttrType, fValue);
		if (err == LASeRNoErr)	return err;

		err = LASeRVideoErr;

		break;
	}

	return err;
}