///////////////////////////////////////////////////////////////////////////////
//
//  This software module was originally developed by SAMSUNG ELECTRONICS CO., LTD.
// and Kyunghee University, Korea in the course of development of the ISO/IEC 
// 14496-20(Lightweight Application Scene Representation -LASeR- and Simple 
// Aggregation Format -SAF-) AMD2: Adaptation standard for reference purposes and
// its performance may not have been optimized.
//
// This software module is an implementation of one or more tools as specified by 
// the ISO/IEC 14496-20 AMD2: Adaptation standard.
//
// ISO/IEC gives users free license to this software module or modifications 
// thereof for use in hardware or software products claiming conformance to 
// audiovisual and image-coding related ITU Recommendations and/or ISO/IEC
// International Standards.
//
// ISO/IEC gives users the same free license to this software module or 
// modifications thereof for research purposes and further ISO/IEC standardisation.
// Those intending to use this software module in hardware or software products are
// advised that its use may infringe existing patents. 
//
// The original developer of this software module and his/her company, the 
// subsequent editors and their companies, and ISO/IEC have no liability for use of
// this software module or modifications thereof in an implementation.
//
// Copyright is not released for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
// The organizations named above retain full rights to use the code for their own
// purpose, assign or donate the code to a third party and inhibit third parties 
// from using the code for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
//
// This copyright notice must be included in all copies or derivative works.
// 
// Copyright (c) ISO/IEC 2010
//
// This notice must be included in all copies or derivative works.
//
#include "..\svg.h"
#include "..\Datatypes.h"


CSVGForeignObject::CSVGForeignObject()
{
	CSVGObject::CSVGObject();

	m_uElementType					= SVG_foreignObject;

	m_pProperties					= new CSVGProperties();
	m_pBasic						= new CSVGBasic();
	m_pFocus						= new CSVGFocus();
	m_pHref							= new CSVGHref();

	m_uFocusHighlight				= 0;
	m_bExternalResourcesRequired	= 0;
	m_fWidth						= 0.0;
	m_fHeight						= 0.0;
	m_fX							= 0.0;
	m_fY							= 0.0;
}

CSVGForeignObject::~CSVGForeignObject()
{
	DELETEMEMORY(m_pProperties);
	DELETEMEMORY(m_pBasic);
	DELETEMEMORY(m_pFocus);
	DELETEMEMORY(m_pHref);
}

/************************************************************************/
// SetAttrs
// Desc		: function of 'foreignObject' to set the children elements or the attributes
// Param	: DOM tree
// History
//	PSH (2008. 11. 19)	- first release
/************************************************************************/
lsrErr CSVGForeignObject::SetAttrs(LPXNode pScene, CLASeR *pI)
{
	lsrErr err = LASeRNoErr;

	if (!m_pProperties || !m_pBasic || !m_pFocus || !m_pHref)
	{
		err = LASeRForeignObjectErr;
		return err;
	}

	const TCHAR *szTemp = NULL;

	/************************************************************************/
	// set attribute
	// svg object
	err = CSVGObject::SetAttrs(pScene, pI);
	CONFIRM(err);

	// svg properties
	err = m_pProperties->SetAttrs(pScene, pI);
	CONFIRM(err);

	// svg basic
	err = m_pBasic->SetAttrs(pScene, pI);
	CONFIRM(err);

	// svg focus
	err = m_pFocus->SetAttrs(pScene, pI);
	CONFIRM(err);

	// svg href
	err = m_pHref->SetAttrs(pScene, pI);
	CONFIRM(err);

	// focusHighlight
	szTemp = pScene->GetAttrValue(_T("focusHighlight"));
	if (szTemp)
	{
		m_uFocusHighlight = ReadFocusHighlightType(szTemp, &err);
		CONFIRM(err);
	}

	// externalResourcesRequired
	szTemp = pScene->GetAttrValue(_T("externalResourcesRequired"));
	if (szTemp)
	{
		m_bExternalResourcesRequired = ReadBooleanType(szTemp, &err);
		CONFIRM(err);
	}

	// width
	szTemp = pScene->GetAttrValue(_T("width"));
	if (szTemp)
		m_fWidth = (float)_tstof(szTemp);

	// height
	szTemp = pScene->GetAttrValue(_T("height"));
	if (szTemp)
		m_fHeight = (float)_tstof(szTemp);

	// x
	szTemp = pScene->GetAttrValue(_T("x"));
	if (szTemp)
		m_fX = (float)_tstof(szTemp);

	// y
	szTemp = pScene->GetAttrValue(_T("y"));
	if (szTemp)
		m_fY = (float)_tstof(szTemp);

	/************************************************************************/
	// 'required' confirmation
	if ((m_fWidth == 0.0) || (m_fHeight == 0.0))
		err = LASeRRectErr;

	return err;
}

/************************************************************************/
// CheckRigthChild
// Desc		: function to check the right child element
// Param	: element type
// History
//	PSH (2008. 11. 21)	- first release
/************************************************************************/
lsrErr CSVGForeignObject::CheckRightChild(UINT8 uElementType)
{
	lsrErr err = LASeRNoErr;

	switch (uElementType)
	{
	case SVG_desc:
	case SVG_metadata:
	case SVG_svg:
	case SVG_switch:
	case SVG_title:
		/* nothing */
		break;

	default:
		err = LASeRForeignObjectErr;
		break;
	}

	return err;
}

/************************************************************************/
// GetAttribute
// Desc		: function of 'foreignObject' to get attribute value using it's type
// Param	: attribute type
// History
//	PSH (2008. 11. 19)	- first release
/************************************************************************/
float CSVGForeignObject::GetFloatAttr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	float fTemp = 0.0;

	switch (uAttrType)
	{
	case ATTR_WIDTH:
		return m_fWidth;
		break;

	case ATTR_HEIGHT:
		return m_fHeight;
		break;

	case ATTR_X:
		return m_fX;
		break;

	case ATTR_Y:
		return m_fY;
		break;

	default:
		fTemp = m_pProperties->GetFloatAttr(uAttrType, pErr);
		if (*pErr == LASeRNoErr)	return fTemp;

		*pErr = LASeRForeignObjectErr;

		break;
	}

	return fTemp;
}

UINT8 CSVGForeignObject::GetUINT8Attr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	UINT8 uTemp = 0;

	switch (uAttrType)
	{
	case ATTR_FOCUSHIGHLIGHT:
		return m_uFocusHighlight;
		break;

	case ATTR_EXTERNALRESOURCESREQUIRED:
		return m_bExternalResourcesRequired;
		break;

	default:
		uTemp = CSVGObject::GetUINT8Attr(uAttrType, pErr);
		if (*pErr == LASeRNoErr)	return uTemp;
		else *pErr = LASeRNoErr;

		uTemp = m_pProperties->GetUINT8Attr(uAttrType, pErr);
		if (*pErr == LASeRNoErr)	return uTemp;
		else *pErr = LASeRNoErr;

		uTemp = m_pFocus->GetUINT8Attr(uAttrType, pErr);
		if (*pErr == LASeRNoErr)	return uTemp;
		else *pErr = LASeRNoErr;

		uTemp = m_pHref->GetUINT8Attr(uAttrType, pErr);
		if (*pErr == LASeRNoErr)	return uTemp;

		*pErr = LASeRForeignObjectErr;

		break;
	}

	return uTemp;
}

tstring CSVGForeignObject::GetStrAttr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	tstring strTemp = _T("");

	strTemp = CSVGObject::GetStrAttr(uAttrType, pErr);
	if (*pErr == LASeRNoErr)	return strTemp;
	else *pErr = LASeRNoErr;

	strTemp = m_pProperties->GetStrAttr(uAttrType, pErr);
	if (*pErr == LASeRNoErr)	return strTemp;
	else *pErr = LASeRNoErr;

	strTemp = m_pBasic->GetStrAttr(uAttrType, pErr);
	if (*pErr == LASeRNoErr)	return strTemp;
	else *pErr = LASeRNoErr;

	strTemp = m_pFocus->GetStrAttr(uAttrType, pErr);
	if (*pErr == LASeRNoErr)	return strTemp;
	else *pErr = LASeRNoErr;

	strTemp = m_pHref->GetStrAttr(uAttrType, pErr);
	if (*pErr == LASeRNoErr)	return strTemp;

	*pErr = LASeRForeignObjectErr;

	return strTemp;
}

deque<UINT8> CSVGForeignObject::GetUINT8ListAttr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	deque<UINT8> auTemp;

	auTemp = m_pProperties->GetUINT8ListAttr(uAttrType, pErr);
	if (*pErr == LASeRNoErr)	return auTemp;
	else *pErr = LASeRNoErr;

	auTemp = m_pBasic->GetUINT8ListAttr(uAttrType, pErr);
	if (*pErr == LASeRNoErr)	return auTemp;

	*pErr = LASeRForeignObjectErr;

	return auTemp;
}

deque<float> CSVGForeignObject::GetFloatListAttr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	deque<float> afTemp;

	afTemp = m_pProperties->GetFloatListAttr(uAttrType, pErr);
	if (*pErr == LASeRNoErr)	return afTemp;

	*pErr = LASeRForeignObjectErr;

	return afTemp;
}

/************************************************************************/
// GetAttrType
// Desc         : function of 'foreignObject' attribute group to get attribute type using it's name
// Param        : attribute name
// History
//	PSH (2008. 11. 19)       - first release
/************************************************************************/
UINT CSVGForeignObject::GetAttrType(tstring strAttrName)
{
	UINT uTemp = 0;

	if (strAttrName == _T("focusHighlight"))					return ATTR_FOCUSHIGHLIGHT;
	else if (strAttrName == _T("externalResourcesRequired"))	return ATTR_WIDTH;
	else if (strAttrName == _T("width"))						return ATTR_WIDTH;
	else if (strAttrName == _T("height"))						return ATTR_HEIGHT;
	else if (strAttrName == _T("x"))							return ATTR_X;
	else if (strAttrName == _T("y"))							return ATTR_Y;
	else
	{
		uTemp = CSVGObject::GetAttrType(strAttrName);
		if (uTemp != 0)	return uTemp;

		uTemp = m_pProperties->GetAttrType(strAttrName);
		if (uTemp != 0)	return uTemp;

		uTemp = m_pBasic->GetAttrType(strAttrName);
		if (uTemp != 0)	return uTemp;

		uTemp = m_pFocus->GetAttrType(strAttrName);
		if (uTemp != 0)	return uTemp;

		uTemp = m_pHref->GetAttrType(strAttrName);
		if (uTemp != 0)	return uTemp;
	}

	return uTemp;
}

/************************************************************************/
// GetAttrValType
// Desc         : function of 'foreignObject' attribute group to get attribute value type using it's type
// Param        : attribute type
// History
//	PSH (2008. 11. 19)       - first release
/************************************************************************/
UINT8 CSVGForeignObject::GetAttrValType(UINT uAttrType)
{
	UINT8 uTemp = 0;

	switch (uAttrType)
	{
	case ATTR_FOCUSHIGHLIGHT:
	case ATTR_EXTERNALRESOURCESREQUIRED:
		return UINT8_TYPE;
		break;

	case ATTR_WIDTH:
	case ATTR_HEIGHT:
	case ATTR_X:
	case ATTR_Y:
		return FLOAT_TYPE;
		break;

	default:
		uTemp = CSVGObject::GetAttrValType(uAttrType);
		if (uTemp != 0)	return uTemp;

		uTemp = m_pProperties->GetAttrValType(uAttrType);
		if (uTemp != 0)	return uTemp;

		uTemp = m_pBasic->GetAttrValType(uAttrType);
		if (uTemp != 0)	return uTemp;

		uTemp = m_pFocus->GetAttrValType(uAttrType);
		if (uTemp != 0)	return uTemp;

		uTemp = m_pHref->GetAttrValType(uAttrType);
		if (uTemp != 0)	return uTemp;

		break;
	}

	return uTemp;
}

/************************************************************************/
// SetAttr
// Desc		: function of 'foreignObject' to set attribute value using it's name
// Param	: target attribute name, value
// History
//	PSH (2008. 11. 19)	- first release
/************************************************************************/
lsrErr CSVGForeignObject::SetAttr(UINT uAttrType, tstring strValue)
{
	lsrErr err = LASeRNoErr;

	const TCHAR *szValue = strValue.c_str();

	switch (uAttrType)
	{
	case ATTR_WIDTH:
		m_fWidth = (float)_tstof(szValue);
		break;

	case ATTR_HEIGHT:
		m_fHeight = (float)_tstof(szValue);
		break;

	case ATTR_X:
		m_fX = (float)_tstof(szValue);
		break;

	case ATTR_Y:
		m_fY = (float)_tstof(szValue);
		break;

	default:
		err = m_pProperties->SetAttr(uAttrType, strValue);
		if (err == LASeRNoErr)	return err;
		else err = LASeRNoErr;

		err = m_pFocus->SetAttr(uAttrType, strValue);
		if (err == LASeRNoErr)	return err;
		else err = LASeRNoErr;

		err = m_pHref->SetAttr(uAttrType, strValue);
		if (err == LASeRNoErr)	return err;

		err = LASeRForeignObjectErr;

		break;
	}

	return err;
}

lsrErr CSVGForeignObject::SetAttr(UINT uAttrType, float fValue)
{
	lsrErr err = LASeRNoErr;

	switch (uAttrType)
	{
	case ATTR_WIDTH:
		m_fWidth = fValue;
		break;

	case ATTR_HEIGHT:
		m_fHeight = fValue;
		break;

	case ATTR_X:
		m_fX = fValue;
		break;

	case ATTR_Y:
		m_fY = fValue;
		break;

	default:
		err = m_pProperties->SetAttr(uAttrType, fValue);
		if (err == LASeRNoErr)	return err;

		err = LASeRForeignObjectErr;

		break;
	}

	return err;
}