///////////////////////////////////////////////////////////////////////////////
//
//  This software module was originally developed by SAMSUNG ELECTRONICS CO., LTD.
// and Kyunghee University, Korea in the course of development of the ISO/IEC 
// 14496-20(Lightweight Application Scene Representation -LASeR- and Simple 
// Aggregation Format -SAF-) AMD2: Adaptation standard for reference purposes and
// its performance may not have been optimized.
//
// This software module is an implementation of one or more tools as specified by 
// the ISO/IEC 14496-20 AMD2: Adaptation standard.
//
// ISO/IEC gives users free license to this software module or modifications 
// thereof for use in hardware or software products claiming conformance to 
// audiovisual and image-coding related ITU Recommendations and/or ISO/IEC
// International Standards.
//
// ISO/IEC gives users the same free license to this software module or 
// modifications thereof for research purposes and further ISO/IEC standardisation.
// Those intending to use this software module in hardware or software products are
// advised that its use may infringe existing patents. 
//
// The original developer of this software module and his/her company, the 
// subsequent editors and their companies, and ISO/IEC have no liability for use of
// this software module or modifications thereof in an implementation.
//
// Copyright is not released for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
// The organizations named above retain full rights to use the code for their own
// purpose, assign or donate the code to a third party and inhibit third parties 
// from using the code for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
//
// This copyright notice must be included in all copies or derivative works.
// 
// Copyright (c) ISO/IEC 2010
//
// This notice must be included in all copies or derivative works.
//
#include "..\svg.h"
#include "..\Datatypes.h"


CSVG::CSVG()
{
	CSVGObject::CSVGObject();

	m_uElementType					= SVG_svg;

	m_pProperties					= new CSVGProperties();
	m_pFocus						= new CSVGFocus();
	m_pAdaptationAttr				= new CLSRAdaptationAttr();

	m_uFocusHighlight				= 0;
	m_bExternalResourcesRequired	= false;
	m_fWidth						= 0.0;
	m_fHeight						= 0.0;
	m_uPreserveAspectRatio			= 0;
	m_uZoomAndPan					= 0;
	m_uSyncBehaviorDefault			= 0;
	m_fSyncToleranceDefault			= 0.0;
	m_strBaseProfile				= _T("");
	m_strContentScriptType			= _T("");
	m_uPlaybackOrder				= 0;
	m_fSnapshotTime					= 0.0;
	m_uTimelineBegin				= 0;
	m_strVersion					= _T("1.0");
	m_bFullscreen					= 0;

	m_uWidthUnit					= 0;
	m_uHeightUnit					= 0;

	apReusedElements.push_back(this);
}

CSVG::~CSVG()
{
	DELETEMEMORY(m_pProperties);
	DELETEMEMORY(m_pFocus);
	DELETEMEMORY(m_pAdaptationAttr);

	m_afViewBox.clear();
}

/************************************************************************/
// SetAttrs
// Desc		: function of 'svg' to set the children elements or the attributes
// Param	: DOM tree
// History
//	PSH (2008. 7. 14)	- first release
//	PSH (2008. 9. 9)	- complete the setting attribute part
/************************************************************************/
lsrErr CSVG::SetAttrs(LPXNode pScene, CLASeR *pI)
{
	lsrErr err = LASeRNoErr;

	if (!m_pProperties || !m_pFocus || !m_pAdaptationAttr)
	{
		err = LASeRSVGErr;
		return err;
	}

	const TCHAR *szTemp = NULL;
	
	/************************************************************************/
	// set attribute
	// svg object
	err = CSVGObject::SetAttrs(pScene, pI);
	CONFIRM(err);

	// svg properties
	err = m_pProperties->SetAttrs(pScene, pI);
	CONFIRM(err);

	// svg focus
	err = m_pFocus->SetAttrs(pScene, pI);
	CONFIRM(err);

	// LASeR adaptationAttr
	err = m_pAdaptationAttr->SetAttrs(pScene, pI);
	CONFIRM(err);

	// focusHighlight
	szTemp = pScene->GetAttrValue(_T("focusHighlight"));
	if (szTemp)
	{
		m_uFocusHighlight = ReadFocusHighlightType(szTemp, &err);
		CONFIRM(err);
	}

	// externalResourcesRequired
	szTemp = pScene->GetAttrValue(_T("externalResourcesRequired"));
	if (szTemp)
	{
		m_bExternalResourcesRequired = ReadBooleanType(szTemp, &err);
		CONFIRM(err);
	}

	// width
	szTemp = pScene->GetAttrValue(_T("width"));
	if (szTemp)
	{
		m_fWidth = ReadValueWithUnitsType(szTemp, &m_uWidthUnit, &err);
		CONFIRM(err);
	}

	// height
	szTemp = pScene->GetAttrValue(_T("height"));
	if (szTemp)
	{
		m_fHeight = ReadValueWithUnitsType(szTemp, &m_uHeightUnit, &err);
		CONFIRM(err);
	}

	// viewBox
	szTemp = pScene->GetAttrValue(_T("viewBox"));
	if (szTemp)
	{
		m_afViewBox = ReadFloatListType(szTemp, ' ', &err);
		CONFIRM(err);

		if (m_afViewBox.size() != 4)			// restriction
		{
			err =  LASeRValidationErr;
			return err;
		}
	}

	// preserveAspectRatio
	szTemp = pScene->GetAttrValue(_T("preserveAspectRatio"));
	if (szTemp)
	{
		m_uPreserveAspectRatio = ReadPreserveAspectRatioType(szTemp, &err);
		CONFIRM(err);
	}

	// zoomAndPan
	szTemp = pScene->GetAttrValue(_T("zoomAndPan"));
	if (szTemp)
	{
		m_uZoomAndPan = ReadZoomAndPanType(szTemp, &err);
		CONFIRM(err);
	}

	// syncBehaviorDefault
	szTemp = pScene->GetAttrValue(_T("syncBehaviorDefault"));
	if (szTemp)
	{
		m_uSyncBehaviorDefault = ReadSyncBehaviorDefaultType(szTemp, &err);
		CONFIRM(err);
	}

	// syncToleranceDefault
	szTemp = pScene->GetAttrValue(_T("syncToleranceDefault"));
	if (szTemp)
		m_fSyncToleranceDefault = (float)_tstof(szTemp);

	// baseProfile
	szTemp = pScene->GetAttrValue(_T("baseProfile"));
	if (szTemp)
		m_strBaseProfile = szTemp;

	// contentScriptType
	szTemp = pScene->GetAttrValue(_T("contentScriptType"));
	if (szTemp)
		m_strContentScriptType = szTemp;

	// playbackOrder
	szTemp = pScene->GetAttrValue(_T("playbackOrder"));
	if (szTemp)
	{
		m_uPlaybackOrder = ReadPlaybackOrderType(szTemp, &err);
		CONFIRM(err);
	}

	// snapshotTime
	szTemp = pScene->GetAttrValue(_T("snapshotTime"));
	if (szTemp)
	{
		m_fSnapshotTime = ReadTimesType(szTemp, &err);
		CONFIRM(err);
	}

	// timelineBegin
	szTemp = pScene->GetAttrValue(_T("timelineBegin"));
	if (szTemp)
	{
		m_uTimelineBegin = ReadTimeLineBeginType(szTemp, &err);
		CONFIRM(err);
	}

	// version
	szTemp = pScene->GetAttrValue(_T("version"));
	if (szTemp)
		m_strVersion = szTemp;

	// lsr:fullscreen
	szTemp = pScene->GetAttrValue(_T("lsr:fullscreen"));
	if (szTemp)
	{
		m_bFullscreen = ReadBooleanType(szTemp, &err);
		CONFIRM(err);
	}

	// set children
	err = AddChildren(pScene);
	CONFIRM(err);

	return err;
}

/************************************************************************/
// CheckRigthChild
// Desc		: function to check the right child element
// Param	: element type
// History
//	PSH (2008. 11. 21)	- first release
/************************************************************************/
lsrErr CSVG::CheckRightChild(UINT8 uElementType)
{
	lsrErr err = LASeRNoErr;

	switch (uElementType)
	{
	case SVG_mpath:
	case SVG_stop:
	case SVG_svg:
	case SVG_tspan:
	case SVGT_fontfacesrc:
	case SVGT_fontfaceuri:
	case SVGT_glyph:
	case SVGT_hkern:
	case SVGT_missingglyph:
	case SVG_tBreak:
		err = LASeRSVGErr;
		return err;
		break;

	default:
		if ((uElementType < SVG_a) || (uElementType > LSR_parsingSwitch))
			err = LASeRSVGErr;

		break;
	}

	return err;
}

/************************************************************************/
// GetAttribute
// Desc		: function of 'svg' to get attribute value using it's type
// Param	: attribute type
// History
//	PSH (2008. 9. 9)	- first release
/************************************************************************/
float CSVG::GetFloatAttr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	float fTemp = 0.0;

	switch (uAttrType)
	{
	case ATTR_WIDTH:
		return m_fWidth;
		break;

	case ATTR_HEIGHT:
		return m_fHeight;
		break;

	case ATTR_SYNCTOLERANCEDEFAULT:
		return m_fSyncToleranceDefault;
		break;

	case ATTR_SNAPSHOTTIME:
		return m_fSnapshotTime;
		break;

	default:
		fTemp = m_pProperties->GetFloatAttr(uAttrType, pErr);
		if (*pErr == LASeRNoErr)	return fTemp;
		
		*pErr = LASeRSVGErr;

		break;
	}

	return fTemp;
}

UINT8 CSVG::GetUINT8Attr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	UINT8 uTemp = 0;

	switch (uAttrType)
	{
	case ATTR_FOCUSHIGHLIGHT:
		return m_uFocusHighlight;
		break;

	case ATTR_EXTERNALRESOURCESREQUIRED:
		return m_bExternalResourcesRequired;
		break;

	case ATTR_PRESERVEASPECTRATIO:
		return m_uPreserveAspectRatio;
		break;

	case ATTR_ZOOMANDPAN:
		return m_uZoomAndPan;
		break;

	case ATTR_SYNCBEHAVIORDEFAULT:
		return m_uSyncBehaviorDefault;
		break;

	case ATTR_PLAYBACKORDER:
		return m_uPlaybackOrder;
		break;

	case ATTR_TIMELINEBEGIN:
		return m_uTimelineBegin;
		break;

	case ATTR_LSR_FULLSCREEN:
		return m_bFullscreen;
		break;

		// custom
	case ATTR_WIDTH_UNIT:
		return m_uWidthUnit;
		break;

	case ATTR_HEIGHT_UNIT:
		return m_uHeightUnit;
		break;

	default:
		uTemp = CSVGObject::GetUINT8Attr(uAttrType, pErr);
		if (*pErr == LASeRNoErr)	return uTemp;
		else *pErr = LASeRNoErr;

		uTemp = m_pProperties->GetUINT8Attr(uAttrType, pErr);
		if (*pErr == LASeRNoErr)	return uTemp;
		else *pErr = LASeRNoErr;

		uTemp = m_pFocus->GetUINT8Attr(uAttrType, pErr);
		if (*pErr == LASeRNoErr)	return uTemp;
		else *pErr = LASeRNoErr;

		uTemp = m_pAdaptationAttr->GetUINT8Attr(uAttrType, pErr);
		if (*pErr == LASeRNoErr)	return uTemp;
		
		*pErr = LASeRSVGErr;

		break;
	}

	return uTemp;
}

UINT CSVG::GetUINTAttr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	UINT uTemp = 0;

	uTemp = m_pAdaptationAttr->GetUINTAttr(uAttrType, pErr);
	if (*pErr == LASeRNoErr)	return uTemp;

	*pErr = LASeRSVGErr;

	return uTemp;
}

tstring CSVG::GetStrAttr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	tstring strTemp = _T("");

	switch (uAttrType)
	{
	case ATTR_BASEPROFILE:
		return m_strBaseProfile;
		break;

	case ATTR_CONTENTSCRIPTTYPE:
		return m_strContentScriptType;
		break;

	case ATTR_VERSION:
		return m_strVersion;
		break;

	default:
		strTemp = CSVGObject::GetStrAttr(uAttrType, pErr);
		if (*pErr == LASeRNoErr)	return strTemp;
		else *pErr = LASeRNoErr;

		strTemp = m_pProperties->GetStrAttr(uAttrType, pErr);
		if (*pErr == LASeRNoErr)	return strTemp;
		else *pErr = LASeRNoErr;

		strTemp = m_pFocus->GetStrAttr(uAttrType, pErr);
		if (*pErr == LASeRNoErr)	return strTemp;
		else *pErr = LASeRNoErr;

		strTemp = m_pAdaptationAttr->GetStrAttr(uAttrType, pErr);
		if (*pErr == LASeRNoErr)	return strTemp;
		
		*pErr = LASeRSVGErr;

		break;
	}

	return strTemp;
}

deque<UINT8> CSVG::GetUINT8ListAttr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	deque<UINT8> auTemp;

	auTemp = m_pProperties->GetUINT8ListAttr(uAttrType, pErr);
	if (*pErr == LASeRNoErr)	return auTemp;
	
	*pErr = LASeRSVGErr;

	return auTemp;
}

deque<float> CSVG::GetFloatListAttr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	deque<float> afTemp;

	switch (uAttrType)
	{
	case ATTR_VIEWBOX:
		return m_afViewBox;
		break;

	default:
		afTemp = m_pProperties->GetFloatListAttr(uAttrType, pErr);
		if (*pErr == LASeRNoErr)	return afTemp;
		else *pErr = LASeRNoErr;

		afTemp = m_pAdaptationAttr->GetFloatListAttr(uAttrType, pErr);
		if (*pErr == LASeRNoErr)	return afTemp;
		
		*pErr = LASeRSVGErr;

		break;
	}

	return afTemp;
}

/************************************************************************/
// GetAttrType
// Desc         : function of 'rect' attribute group to get attribute type using it's name
// Param        : attribute name
// History
//	PSH (2008. 8. 26)       - first release
/************************************************************************/
UINT CSVG::GetAttrType(tstring strAttrName)
{
	UINT uTemp = 0;

	if (strAttrName == _T("focusHighlight"))					return ATTR_FOCUSHIGHLIGHT;
	else if (strAttrName == _T("externalResourcesRequired"))	return ATTR_EXTERNALRESOURCESREQUIRED;
	else if (strAttrName == _T("width"))						return ATTR_WIDTH;
	else if (strAttrName == _T("height"))						return ATTR_HEIGHT;
	else if (strAttrName == _T("viewBox"))						return ATTR_VIEWBOX;
	else if (strAttrName == _T("preserveAspectRatio"))			return ATTR_PRESERVEASPECTRATIO;
	else if (strAttrName == _T("zoomAndPan"))					return ATTR_ZOOMANDPAN;
	else if (strAttrName == _T("syncBehaviorDefault"))			return ATTR_SYNCBEHAVIORDEFAULT;
	else if (strAttrName == _T("syncToleranceDefault"))			return ATTR_SYNCTOLERANCEDEFAULT;
	else if (strAttrName == _T("baseProfile"))					return ATTR_BASEPROFILE;
	else if (strAttrName == _T("contentScriptType"))			return ATTR_CONTENTSCRIPTTYPE;
	else if (strAttrName == _T("playbackOrder"))				return ATTR_PLAYBACKORDER;
	else if (strAttrName == _T("snapshotTime"))					return ATTR_SNAPSHOTTIME;
	else if (strAttrName == _T("timelineBegin"))				return ATTR_TIMELINEBEGIN;
	else if (strAttrName == _T("version"))						return ATTR_VERSION;
	else if (strAttrName == _T("lsr:fullscreen"))				return ATTR_LSR_FULLSCREEN;
	else
	{
		uTemp = CSVGObject::GetAttrType(strAttrName);
		if (uTemp != 0)	return uTemp;

		uTemp = m_pProperties->GetAttrType(strAttrName);
		if (uTemp != 0)	return uTemp;

		uTemp = m_pFocus->GetAttrType(strAttrName);
		if (uTemp != 0)	return uTemp;

		uTemp = m_pAdaptationAttr->GetAttrType(strAttrName);
		if (uTemp != 0)	return uTemp;
	}

	return uTemp;
}

/************************************************************************/
// GetAttrValType
// Desc         : function of 'rect' attribute group to get attribute value type using it's type
// Param        : attribute type
// History
//	PSH (2008. 11. 8)       - first release
/************************************************************************/
UINT8 CSVG::GetAttrValType(UINT uAttrType)
{
	UINT8 uTemp = 0;

	switch (uAttrType)
	{
	case ATTR_FOCUSHIGHLIGHT:
	case ATTR_EXTERNALRESOURCESREQUIRED:
	case ATTR_PRESERVEASPECTRATIO:
	case ATTR_ZOOMANDPAN:
	case ATTR_SYNCBEHAVIORDEFAULT:
	case ATTR_PLAYBACKORDER:
	case ATTR_TIMELINEBEGIN:
	case ATTR_LSR_FULLSCREEN:
		return UINT8_TYPE;
		break;

	case ATTR_WIDTH:
	case ATTR_HEIGHT:
	case ATTR_SYNCTOLERANCEDEFAULT:
	case ATTR_SNAPSHOTTIME:
		return FLOAT_TYPE;
		break;

	case ATTR_VIEWBOX:
		return FLOAT_LIST_TYPE;
		break;

	case ATTR_BASEPROFILE:
	case ATTR_CONTENTSCRIPTTYPE:
	case ATTR_VERSION:
		return STRING_TYPE;
		break;

	default:
		uTemp = CSVGObject::GetAttrValType(uAttrType);
		if (uTemp != 0)	return uTemp;

		uTemp = m_pProperties->GetAttrValType(uAttrType);
		if (uTemp != 0)	return uTemp;

		uTemp = m_pFocus->GetAttrValType(uAttrType);
		if (uTemp != 0)	return uTemp;

		uTemp = m_pAdaptationAttr->GetAttrValType(uAttrType);
		if (uTemp != 0)	return uTemp;

		break;
	}

	return uTemp;
}

/************************************************************************/
// SetAttr
// Desc		: function of 'svg' to set attribute value using it's name
// Param	: target attribute name, value
// History
//	PSH (2008. 9. 9)	- first release
/************************************************************************/
lsrErr CSVG::SetAttr(UINT uAttrType, tstring strValue)
{
	lsrErr err = LASeRNoErr;

	switch (uAttrType)
	{
	case ATTR_WIDTH:
		m_fWidth = ReadValueWithUnitsType(strValue.c_str(), &m_uWidthUnit, &err);
		CONFIRM(err);
		break;

	case ATTR_HEIGHT:
		m_fHeight = ReadValueWithUnitsType(strValue.c_str(), &m_uHeightUnit, &err);
		CONFIRM(err);
		break;

	default:
		err = m_pProperties->SetAttr(uAttrType, strValue);
		if (err == LASeRNoErr)	return err;
		else err = LASeRNoErr;

		err = m_pFocus->SetAttr(uAttrType, strValue);
		if (err == LASeRNoErr)	return err;

		err = LASeRSVGErr;

		break;
	}

	return err;
}